#ifndef AUDIOOUTPUTUTIL
#define AUDIOOUTPUTUTIL

using namespace std;

#define IIR_NUM_COEFFICIENTS				(3)	// Assumed biquads

class AudioOutputUtil
{
 public:
    
    //Notch Filter / IRR transformation
    // Parameters
    typedef struct
    {
        uint N;
        uint num_biquads;
        float b[IIR_NUM_COEFFICIENTS];
        float a[IIR_NUM_COEFFICIENTS];
    } IIR_Parameters;
    
    // State information
    typedef struct
    {
        float a[IIR_NUM_COEFFICIENTS];
        float b[IIR_NUM_COEFFICIENTS];
        float x_mem[IIR_NUM_COEFFICIENTS];
        float y_mem[IIR_NUM_COEFFICIENTS];
    } IIR_State;
    
    // Instance
    typedef struct
    {
        IIR_Parameters parameters;
        IIR_State state;
    } IIR;
    
    // Parameters
    typedef struct
    {
        uint N;
        float frequency;			// Notch frequency
        float attenuation_dB;		// Notch attenuation
    } Notch_Parameters;

    // State information

    typedef struct
    {
        float pole_position;
        float frequency_offset;
        float a1s;
        float a2;
        float depth_factor;
        float w0;		// Input weight
        float w1;		// Output weight
    } Notch_State;

    // Instance
    typedef struct
    {
        Notch_Parameters parameters;
        Notch_State state;
        IIR iir;
    } Notch;

    static void notch_update(Notch &instance, float fs);
    static void notch_execute(Notch &instance, float *in, float *out);
};

#endif
